package com.utopiaindustries.qualitychecker.db;

import android.annotation.SuppressLint;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import com.utopiaindustries.qualitychecker.models.InspectionReportWrapper;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.RejectedExecutionException;

public class ReportRepository {

    private final DatabaseHelper dbHelper;
    private SQLiteDatabase database;
    private final ExecutorService executorService;

    public ReportRepository(Context context) {
        dbHelper = new DatabaseHelper(context);
        executorService = Executors.newSingleThreadExecutor();
    }

    private void openDatabase() {
        if (database == null || !database.isOpen()) {
            database = dbHelper.getWritableDatabase();
        }
    }

    public void insert(InspectionReportWrapper wrapper) {
        executeSafely(() -> {
            openDatabase();
            database.beginTransaction();
            try {
                ContentValues values = new ContentValues();
                values.put(DatabaseHelper.REPORT_COLUMN_CONTENT, wrapper.getContent());
                values.put(DatabaseHelper.REPORT_COLUMN_SYNCED, wrapper.isSynced());
                database.insertWithOnConflict(DatabaseHelper.REPORT_TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
                database.setTransactionSuccessful();
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                database.endTransaction();
                close();
            }
        });
    }

    public void updateSyncStatus(long id) {
        executeSafely(() -> {
            openDatabase();
            database.beginTransaction();
            try {
                ContentValues values = new ContentValues();
                values.put(DatabaseHelper.REPORT_COLUMN_SYNCED, true );  // Set the synced value to true
                String whereClause = DatabaseHelper.REPORT_COLUMN_ID + " = ?";
                String[] whereArgs = {String.valueOf(id)};
                database.update(DatabaseHelper.REPORT_TABLE_NAME, values, whereClause, whereArgs);
                database.setTransactionSuccessful();
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                database.endTransaction();
                close();
            }
        });
    }

    @SuppressLint("Range")
    public List<InspectionReportWrapper> findAllUnsynced() {
        List<InspectionReportWrapper> reports = new ArrayList<>();
        openDatabase();
        String selection = DatabaseHelper.REPORT_COLUMN_SYNCED + " = ?";
        String[] selectionArgs = {"0"};

        try (Cursor cursor = database.query(DatabaseHelper.REPORT_TABLE_NAME,
                null,
                selection,
                selectionArgs,
                null,
                null,
                null)) {

            if (cursor != null && cursor.moveToFirst()) {
                do {
                    InspectionReportWrapper wrapper = new InspectionReportWrapper();
                    wrapper.setId(cursor.getLong(cursor.getColumnIndex(DatabaseHelper.REPORT_COLUMN_ID)));
                    wrapper.setContent(cursor.getString(cursor.getColumnIndex(DatabaseHelper.REPORT_COLUMN_CONTENT)));
                    wrapper.setSynced(cursor.getInt(cursor.getColumnIndex(DatabaseHelper.REPORT_COLUMN_SYNCED)) == 1);
                    reports.add(wrapper);
                } while (cursor.moveToNext());
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            close();
        }
        return reports;
    }

    /*
     *  Close the database when all operations are complete
     */
    public void close() {
        if (database != null && database.isOpen()) {
            database.close();
        }
    }

    /*
     *  Executes a task safely, ensuring the ExecutorService is still active
     */
    private void executeSafely(Runnable task) {
        try {
            if (!executorService.isShutdown()) {
                executorService.submit(task);
            } else {
                // Handle the situation when the executor is already shut down
                System.err.println("Task execution rejected because ExecutorService is shut down");
            }
        } catch (RejectedExecutionException e) {
            e.printStackTrace();
        }
    }
}
